<?php
	require_once('data.config.php');
	
	// request json data from other server to avoid content security blocks
	if ( !empty($_GET['u']) ) {
		$headers = getallheaders();
		$ch = curl_init($_GET['u']);
		curl_setopt($ch, CURLOPT_FRESH_CONNECT, true);
		curl_setopt($ch, CURLOPT_FORBID_REUSE, true);
		curl_setopt($ch, CURLOPT_TIMEOUT, 10);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		if ( !empty($headers['Authorization']) ) {
			curl_setopt($ch, CURLOPT_HTTPHEADER, ['Authorization: '. $headers['Authorization']]);
		}
		$jsonData = curl_exec($ch);
		curl_close($ch);
	}

	else {
		// check token
		$headers = getallheaders();
		if ( empty($headers['Authorization']) or $headers['Authorization'] != 'Bearer '. $CONFIG['token']) {
			die('Authorization failed.');
		}
		
		$system = new System();
		$result = [];

		foreach($CONFIG['charts'] as $chartId => $chartOptions) {
			if ( !empty($chartOptions) ) {
				$result['charts'][ $chartId ] = $system->get($chartId, $chartOptions);
			}
		}

		$result['timestamp'] = time();
		$result['version'] = 2;
		$result['os'] = strtolower(PHP_OS);
		
		$jsonData = json_encode($result);
	}
	
	header('Content-Type: application/json');
	echo $jsonData;


class System {

	public function get( $name, $options = [] ) {
		$os = strtolower(PHP_OS);
		$result = [];
		
		if ( $os == 'winnt' ) {
			switch ( trim($name) ) {
				case 'load':
					$result = $this->getWindowsServerLoad();
					break;
				case 'disks':
					$result = $this->getWindowsDiskStatus();
					break;
				case 'volumes':
					$result = $this->getWindowsVolumeStatus();
					break;
				case 'memory':
					$result = $this->getWindowsMemory();
					break;
				case 'processes':
					$result = $this->getWindowsProcesses($options);
					break;
			}
		}
		elseif ( $os == 'linux' ) {
			switch ( trim($name) ) {
				case 'load':
					$result = $this->getLinuxServerLoad();
					break;
				case 'disks':
					$result = $this->getLinuxDiskStatus();
					break;
				case 'volumes':
					$result = $this->getLinuxVolumeStatus();
					break;
				case 'memory':
					$result = $this->getLinuxMemory();
					break;
				case 'processes':
					$result = $this->getLinuxProcesses($options);
					break;
			}
		}
		else {
			die(PHP_OS .' is not supported');
		}
		
		return $result;
	}
	
	
	/***
	* LINUX
	*/
	private function getLinuxServerLoad() {
		$file = '/proc/loadavg';
		if ( is_readable($file) ) {
			$data = file_get_contents($file);
	    $items = explode(' ', $data);
	  }
	  else {
	  	$items = sys_getloadavg();
	  }
    
    $result = [
    	'data' => [
    		'1' => (double) $items[0],
    		'5' => (double) $items[1],
    		'15' => (double) $items[2]
    	]
    ];

		return $result;
	}
	
	
	private function getLinuxDiskStatus() {
		$result = ['data' => []];
		$file = '/proc/mdstat';

		if ( file_exists($file) ) {
			$data = file_get_contents($file);
			if ( 
				preg_match_all('/(md[0-9])/', $data, $matches1, PREG_PATTERN_ORDER) and
				preg_match_all('/\[([U_]{2,})\]/m', $data, $matches2, PREG_PATTERN_ORDER) and
				count($matches1[1]) == count($matches2[1])
			) {
				for($i = 0; $i < count($matches1); $i++) {
					$name = $matches1[1][$i];
					$status = ( strpos($matches2[1][$i], '_') === false )? 'OK' : 'ERROR';
					$result['data'][ $name ] = $status;
				}
			}
		}
    
		return $result;
	}
	
	
	private function getLinuxVolumeStatus() {
		@exec('df', $output);
		array_shift($output);
		$result = ['data' => [], 'unit' => 'GB'];
		
		foreach($output as $row) {
			$row =  preg_replace('!\s+!', ' ', $row);
			$items = explode(' ', $row);
			if ( $items[0] != 'tmpfs') {
				$result['data'][$items[0]] = [
					'used' => (double) round($items[2] / (1024 * 1024), 2),
					'free' => (double) round($items[3] / (1024 * 1024), 2)
				];
			}
		}

		return $result;
	}
	
	
	private function getLinuxMemory() {
		@exec('free', $output);
		$row = preg_replace('!\s+!', ' ', $output[1]);
		$items = explode(' ', $row);

		$result = [
			'data' => [
				'used' => (double) round($items[2] / (1024 * 1024), 2),
				'cache' => (double) round($items[5] / (1024 * 1024), 2),
				'free' => (double) round($items[3] / (1024 * 1024), 2)
			],
			'unit' => 'GB'
		];

		return $result;
	}
	
	
	private function getLinuxProcesses( $options ) {
		$result = [ 'data' => [] ];

		foreach($options as $option) {
			$output = [];
			@exec('ps -efa | grep '. $option, $output);
			$result['data'][$option] = count($output) - 2;
		}

		return $result;
	}
	
	
	private function getLinuxServerName() {
		$result = file_get_contents('/proc/sys/kernel/hostname');
		$result = str_replace("\n", "", $result);
		return $result;
	}
	
	
	
	/***
	* WINDOWS
	*/

	public function getWindowsServerLoad() {
		$cmd = "wmic CPU get LoadPercentage";
    @exec($cmd, $output);

		$load = $this->getWmicResult($output, 'LoadPercentage');
		$result = [
			'unit' => '%',
			'data' => [
				'0' => (int) $load
			]
		];
		
		return $result;
	}
	

	public function getWindowsDiskStatus() {
		$cmd = "wmic DiskDrive get Caption,Status";
		@exec($cmd, $output);
		$disks = $this->getWmicResult($output);
		
		$result = ['data' => []];

		foreach($disks as $disk) {
			$result['data'][] = [
				'name' => $disk['Caption'],
				'status' => $disk['Status']
			];
		}

		return $result;
	}
	
	
	public function getWindowsMemory() {
		$cmd = "wmic OS get FreePhysicalMemory,TotalVisibleMemorySize";
		@exec($cmd, $output);
		$memory = $this->getWmicResult($output);
		
		$totalMemory = $memory[0]['TotalVisibleMemorySize'];
		$freeMemory = $memory[0]['FreePhysicalMemory'];

		$result = [
			'data' => [
				'free' => (double) round($freeMemory / (1024 * 1024), 2),
				'used' => (double) round(($totalMemory - $freeMemory) / (1024 * 1024), 2),
			],
			'unit' => 'GB'
		];

		return $result;
	}
	
	
	public function getWindowsVolumeStatus() {
		$cmd = "wmic volume Get capacity,freespace,label,driveletter";
		@exec($cmd, $output);
		
		$result = ['data' => [], 'unit' => 'GB'];
		$gb = 1024 * 1024 * 1024;

		$volumes = $this->getWmicResult($output);
		foreach($volumes as $volume) {
			if ( $volume['DriveLetter'] != '' ) {
				$name = ( $volume['Label'] != '' )? $volume['Label'] .' ('. $volume['DriveLetter'] .')' : $volume['DriveLetter'];
				$result['data'][$name] = [
					'used' => (double) round(($volume['Capacity'] - $volume['FreeSpace']) / $gb, 2),
					'free' => (double) round($volume['FreeSpace'] / $gb, 2)
				];
			}
		}

		return $result;
	}
	
	
	private function getWindowsProcesses( $options ) {
		$result = [ 'data' => [] ];

		foreach($options as $option) {
			$output = [];
			@exec('wmic process get Description | grep '. $option, $output);
			$result['data'][$option] = (int) count($output) - 1;
		}

		return $result;
	}
	
	
	public function getWindowsServerName() {
		$cmd = "wmic ComputerSystem get Caption";
		@exec($cmd, $output);
		
		$result = $this->getWmicResult($output, 'Caption');
		
		return $result;
	}
	
	
	private function getWmicResult( $output, $key = '' ) {
		$data = [];
		$keys = [];
		$pos = [0];
		
		// get header
		$header = array_shift($output);
		$lastPos = 0;
		for($i = 1; $i < strlen($header); $i++) {
			if ( ($header[$i - 1] == ' ' and $header[$i] != ' ') ) {
				$keys[] = trim( substr($header, $lastPos, $i - $lastPos) );
				$pos[] = $i;
				$lastPos = $i;
			}
		}
		$keys[] = substr($header, $lastPos);
		
		// get content
		foreach($output as $line) {
			if ( trim($line) != '' ) {
				$row = [];
				for($i = 0; $i < count($pos); $i++) {
					$lastPos = ( $i < count($pos) - 1 )? $pos[$i + 1] : strlen($line);
					$row[ $keys[$i] ] = trim( substr($line, $pos[$i], $lastPos - $pos[$i]) );
				}
				$data[] = $row;
			}
		}
		
		// get value for a specific key
		if ( $key != '' ) {
			$result = '';
			foreach($data as $row) {
				if ( isset($row[ $key ]) ) {
					$result = $row[ $key ];
					break;
				}
			}
		}
		else {
			$result = $data;
		}

		return $result;
	}
	
}

?>